@@ -37,6 +37,7 @@
 #include "pp-utils.h"
 
 #include <dbus/dbus-glib.h>
+#include <libnotify/notify.h>
 
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
@@ -46,7 +47,16 @@
 
 #define PACKAGE_KIT_BUS "org.freedesktop.PackageKit"
 #define PACKAGE_KIT_PATH "/org/freedesktop/PackageKit"
-#define PACKAGE_KIT_IFACE "org.freedesktop.PackageKit.Modify"
+#define PACKAGE_KIT_MODIFY_IFACE "org.freedesktop.PackageKit.Modify"
+#define PACKAGE_KIT_QUERY_IFACE  "org.freedesktop.PackageKit.Query"
+
+#define FIREWALLD_BUS "org.fedoraproject.FirewallD"
+#define FIREWALLD_PATH "/org/fedoraproject/FirewallD"
+#define FIREWALLD_IFACE "org.fedoraproject.FirewallD"
+
+#define SCP_BUS   "org.fedoraproject.Config.Printing"
+#define SCP_PATH  "/org/fedoraproject/Config/Printing"
+#define SCP_IFACE "org.fedoraproject.Config.Printing"
 
 #define ALLOWED_CHARACTERS "abcdefghijklmnopqrtsuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_/"
 
@@ -57,7 +67,6 @@ enum
 {
   NOTEBOOK_LOCAL_PAGE = 0,
   NOTEBOOK_NETWORK_PAGE,
-  NOTEBOOK_HP_JETDIRECT_PAGE,
   NOTEBOOK_N_PAGES
 };
 
@@ -80,7 +89,6 @@ enum
 {
   DEVICE_TYPE_LOCAL = 0,
   DEVICE_TYPE_NETWORK,
-  DEVICE_TYPE_HP_JETDIRECT,
   DEVICE_TYPE_N
 };
 
@@ -115,8 +123,30 @@ struct _PpNewPrinterDialog {
   gpointer             user_data;
 
   GCancellable *cancellable;
+
+  gchar    *warning;
+  gboolean  show_warning;
 };
 
+static void
+show_notification (gchar *primary_text,
+                   gchar *secondary_text,
+                   gchar *icon_name)
+{
+  if (primary_text)
+    {
+      NotifyNotification *notification;
+      notification = notify_notification_new (primary_text,
+                                              secondary_text,
+                                              icon_name);
+      notify_notification_set_app_name (notification, _("Printers"));
+      notify_notification_set_hint (notification, "transient", g_variant_new_boolean (TRUE));
+
+      notify_notification_show (notification, NULL);
+      g_object_unref (notification);
+    }
+}
+
 static void
 device_type_selection_changed_cb (GtkTreeSelection *selection,
                                   gpointer          user_data)
@@ -144,7 +174,10 @@ device_type_selection_changed_cb (GtkTreeSelection *selection,
       widget = (GtkWidget*)
         gtk_builder_get_object (pp->builder, "device-type-notebook");
 
-      gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), device_type);
+      if (pp->show_warning && device_type == DEVICE_TYPE_NETWORK)
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), 2);
+      else
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), device_type);
 
       if (device_type == DEVICE_TYPE_LOCAL)
         treeview = (GtkWidget*)
@@ -268,16 +301,19 @@ devices_get_cb (GObject      *source_object,
   PpNewPrinterDialog *pp = user_data;
   cups_dest_t        *dests;
   GHashTable         *devices = NULL;
+  GDBusProxy         *proxy;
   GtkWidget          *widget = NULL;
   GVariant           *dg_output = NULL;
   gboolean            already_present;
   GError             *error = NULL;
   gchar              *new_name = NULL;
+  gchar              *device_uri = NULL;
   char               *ret_error = NULL;
-  gint                i, j;
+  gint                i, j, k;
   gint                name_index;
   gint                num_dests;
 
+
   dg_output = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object),
                                         res,
                                         &error);
@@ -417,22 +453,113 @@ devices_get_cb (GObject      *source_object,
                * Show devices with device-id.
                * Other preferences should apply here.
                */
-              for (i = 0; i < pp->num_devices; i++)
+              proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                                     G_DBUS_PROXY_FLAGS_NONE,
+                                                     NULL,
+                                                     SCP_BUS,
+                                                     SCP_PATH,
+                                                     SCP_IFACE,
+                                                     NULL,
+                                                     &error);
+
+              if (proxy)
                 {
-                  for (j = 0; j < pp->num_devices; j++)
+                  GVariantBuilder  device_list;
+                  GVariantBuilder  device_hash;
+                  GVariant        *input = NULL;
+                  GVariant        *output = NULL;
+                  GVariant        *array = NULL;
+                  GVariant        *subarray = NULL;
+
+                  g_variant_builder_init (&device_list, G_VARIANT_TYPE ("a{sv}"));
+
+                  for (i = 0; i < pp->num_devices; i++)
                     {
-                      if (i != j)
+                      if (pp->devices[i].device_uri)
                         {
-                          if (g_strcmp0 (pp->devices[i].display_name, pp->devices[j].display_name) == 0)
+                          g_variant_builder_init (&device_hash, G_VARIANT_TYPE ("a{ss}"));
+
+                          if (pp->devices[i].device_id)
+                            g_variant_builder_add (&device_hash,
+                                                   "{ss}",
+                                                   "device-id",
+                                                   pp->devices[i].device_id);
+
+                          if (pp->devices[i].device_make_and_model)
+                            g_variant_builder_add (&device_hash,
+                                                   "{ss}",
+                                                   "device-make-and-model",
+                                                   pp->devices[i].device_make_and_model);
+
+                          if (pp->devices[i].device_class)
+                            g_variant_builder_add (&device_hash,
+                                                   "{ss}",
+                                                   "device-class",
+                                                   pp->devices[i].device_class);
+
+                          g_variant_builder_add (&device_list,
+                                                 "{sv}",
+                                                 pp->devices[i].device_uri,
+                                                 g_variant_builder_end (&device_hash));
+                        }
+                    }
+
+                  input = g_variant_new ("(v)", g_variant_builder_end (&device_list));
+
+                  output = g_dbus_proxy_call_sync (proxy,
+                                                   "GroupPhysicalDevices",
+                                                   input,
+                                                   G_DBUS_CALL_FLAGS_NONE,
+                                                   60000,
+                                                   NULL,
+                                                   &error);
+
+                  if (output && g_variant_n_children (output) == 1)
+                    {
+                      array = g_variant_get_child_value (output, 0);
+                      if (array)
+                        {
+                          for (i = 0; i < g_variant_n_children (array); i++)
                             {
-                              if (pp->devices[i].device_id && !pp->devices[j].show)
+                              subarray = g_variant_get_child_value (array, i);
+                              if (subarray)
                                 {
-                                  pp->devices[i].show = TRUE;
+                                  device_uri = g_strdup (g_variant_get_string (
+                                                 g_variant_get_child_value (subarray, 0),
+                                                 NULL));
+
+                                  for (k = 0; k < pp->num_devices; k++)
+                                    if (g_str_has_prefix (pp->devices[k].device_uri, device_uri))
+                                      pp->devices[k].show = TRUE;
+
+                                  g_free (device_uri);
                                 }
                             }
                         }
                     }
+
+                  if (output)
+                    g_variant_unref (output);
+                  g_variant_unref (input);
+                  g_object_unref (proxy);
                 }
+
+              if (error)
+                {
+                  if (proxy == NULL ||
+                      (error->domain == G_DBUS_ERROR &&
+                       (error->code == G_DBUS_ERROR_SERVICE_UNKNOWN ||
+                        error->code == G_DBUS_ERROR_UNKNOWN_METHOD)))
+                    g_warning ("Install system-config-printer which provides \
+DBus method \"GroupPhysicalDevices\" to group duplicates in device list.");
+
+                  for (i = 0; i < pp->num_devices; i++)
+                    pp->devices[i].show = TRUE;
+                }
+
+              for (i = 0; i < pp->num_devices; i++)
+                if (!pp->devices[i].device_id)
+                  pp->devices[i].show = FALSE;
             }
 
           g_hash_table_destroy (devices);
@@ -479,6 +606,13 @@ devices_get (PpNewPrinterDialog *pp)
 
   if (proxy)
     {
+      if (pp->show_warning)
+        {
+          widget = (GtkWidget*)
+            gtk_builder_get_object (pp->builder, "device-type-notebook");
+          gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), 2);
+        }
+
       in_include = g_variant_builder_new (G_VARIANT_TYPE ("as"));
       in_exclude = g_variant_builder_new (G_VARIANT_TYPE ("as"));
 
@@ -598,6 +732,180 @@ line_split (gchar *line)
   return result;
 }
 
+static void
+service_enable (gchar *service_name,
+                gint   service_timeout)
+{
+  GDBusProxy *proxy;
+  GVariant   *input = NULL;
+  GVariant   *output = NULL;
+  GError     *error = NULL;
+  gint        result;
+
+  proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                         G_DBUS_PROXY_FLAGS_NONE,
+                                         NULL,
+                                         FIREWALLD_BUS,
+                                         FIREWALLD_PATH,
+                                         FIREWALLD_IFACE,
+                                         NULL,
+                                         &error);
+
+  if (proxy)
+    {
+      input = g_variant_new ("(si)",
+                             service_name,
+                             service_timeout);
+
+      output = g_dbus_proxy_call_sync (proxy,
+                                       "enableService",
+                                       input,
+                                       G_DBUS_CALL_FLAGS_NONE,
+                                       60000,
+                                       NULL,
+                                       &error);
+
+      if (output && g_variant_n_children (output) == 1)
+        g_variant_get (output, "(i)", &result);
+
+      if (output)
+        g_variant_unref (output);
+      g_variant_unref (input);
+      g_object_unref (proxy);
+    }
+}
+
+static void
+service_disable (gchar *service_name)
+{
+  GDBusProxy *proxy;
+  GVariant   *input = NULL;
+  GVariant   *output = NULL;
+  GError     *error = NULL;
+  gint        result;
+
+  proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                         G_DBUS_PROXY_FLAGS_NONE,
+                                         NULL,
+                                         FIREWALLD_BUS,
+                                         FIREWALLD_PATH,
+                                         FIREWALLD_IFACE,
+                                         NULL,
+                                         &error);
+
+  if (proxy)
+    {
+      input = g_variant_new ("(s)", service_name);
+
+      output = g_dbus_proxy_call_sync (proxy,
+                                       "disableService",
+                                       input,
+                                       G_DBUS_CALL_FLAGS_NONE,
+                                       60000,
+                                       NULL,
+                                       &error);
+
+      if (output && g_variant_n_children (output) == 1)
+        g_variant_get (output, "(i)", &result);
+
+      if (output)
+        g_variant_unref (output);
+      g_variant_unref (input);
+      g_object_unref (proxy);
+    }
+}
+
+static gboolean
+service_enabled (gchar *service_name)
+{
+  GDBusProxy *proxy;
+  GVariant   *input = NULL;
+  GVariant   *output = NULL;
+  GError     *error = NULL;
+  gint        query_result = 0;
+
+  proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                         G_DBUS_PROXY_FLAGS_NONE,
+                                         NULL,
+                                         FIREWALLD_BUS,
+                                         FIREWALLD_PATH,
+                                         FIREWALLD_IFACE,
+                                         NULL,
+                                         &error);
+
+  if (proxy)
+    {
+      input = g_variant_new ("(s)",
+                             service_name);
+
+      output = g_dbus_proxy_call_sync (proxy,
+                                       "queryService",
+                                       input,
+                                       G_DBUS_CALL_FLAGS_NONE,
+                                       60000,
+                                       NULL,
+                                       &error);
+
+      if (output && g_variant_n_children (output) == 1)
+        g_variant_get (output, "(i)", &query_result);
+
+      if (output)
+        g_variant_unref (output);
+      g_variant_unref (input);
+      g_object_unref (proxy);
+    }
+
+  if (query_result > 0)
+    return TRUE;
+  else
+    return FALSE;
+}
+
+static gboolean
+dbus_method_available (gchar *name,
+                       gchar *path,
+                       gchar *iface,
+                       gchar *method)
+{
+  GDBusProxy *proxy;
+  GError     *error = NULL;
+  GVariant   *output = NULL;
+  gboolean    result = FALSE;
+
+  proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                         G_DBUS_PROXY_FLAGS_NONE,
+                                         NULL,
+                                         name,
+                                         path,
+                                         iface,
+                                         NULL,
+                                         NULL);
+
+  if (proxy)
+    {
+      output = g_dbus_proxy_call_sync (proxy,
+                                       method,
+                                       NULL,
+                                       G_DBUS_CALL_FLAGS_NONE,
+                                       60000,
+                                       NULL,
+                                       &error);
+
+      if (error &&
+          error->domain == G_DBUS_ERROR &&
+          error->code == G_DBUS_ERROR_SERVICE_UNKNOWN)
+        result = FALSE;
+      else
+        result = TRUE;
+
+      if (output)
+        g_variant_unref (output);
+      g_object_unref (proxy);
+    }
+
+  return result;
+}
+
 static void
 search_address_cb (GtkToggleButton *togglebutton,
                    gpointer         user_data)
@@ -825,10 +1133,14 @@ actualize_devices_list (PpNewPrinterDialog *pp)
 {
   GtkListStore *network_store;
   GtkListStore *local_store;
+  GtkTreeModel *model;
   GtkTreeView  *network_treeview;
   GtkTreeView  *local_treeview;
   GtkTreeIter   iter;
+  GtkWidget    *treeview;
+  GtkWidget    *widget;
   gint          i;
+  gint          device_type = -1;
 
   network_treeview = (GtkTreeView*)
     gtk_builder_get_object (pp->builder, "network-devices-treeview");
@@ -846,7 +1158,8 @@ actualize_devices_list (PpNewPrinterDialog *pp)
 
   for (i = 0; i < pp->num_devices; i++)
     {
-      if (pp->devices[i].device_id || pp->devices[i].device_ppd_uri)
+      if ((pp->devices[i].device_id || pp->devices[i].device_ppd_uri) &&
+          pp->devices[i].show)
         {
           if (g_strcmp0 (pp->devices[i].device_class, "network") == 0)
             {
@@ -855,6 +1168,7 @@ actualize_devices_list (PpNewPrinterDialog *pp)
                                   DEVICE_ID_COLUMN, i,
                                   DEVICE_NAME_COLUMN, pp->devices[i].display_name,
                                   -1);
+              pp->show_warning = FALSE;
             }
           else if (g_strcmp0 (pp->devices[i].device_class, "direct") == 0)
             {
@@ -880,6 +1194,23 @@ actualize_devices_list (PpNewPrinterDialog *pp)
       gtk_tree_view_get_selection (GTK_TREE_VIEW (local_treeview)),
       &iter);
 
+  treeview = (GtkWidget*)
+    gtk_builder_get_object (pp->builder, "device-types-treeview");
+
+  if (gtk_tree_selection_get_selected (
+        gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)), &model, &iter))
+    gtk_tree_model_get (model, &iter,
+                        DEVICE_TYPE_TYPE_COLUMN, &device_type,
+                        -1);
+
+  widget = (GtkWidget*)
+    gtk_builder_get_object (pp->builder, "device-type-notebook");
+
+  if (pp->show_warning && device_type == DEVICE_TYPE_NETWORK)
+    gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), 2);
+  else
+    gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), device_type);
+
   g_object_unref (network_store);
   g_object_unref (local_store);
 }
@@ -889,6 +1220,9 @@ populate_devices_list (PpNewPrinterDialog *pp)
 {
   GtkTreeViewColumn *column;
   GtkCellRenderer   *renderer;
+  GtkTextBuffer     *text_buffer;
+  GtkTextView       *warning_textview;
+  GtkTextIter        text_iter;
   GtkWidget         *network_treeview;
   GtkWidget         *local_treeview;
 
@@ -905,6 +1239,41 @@ populate_devices_list (PpNewPrinterDialog *pp)
                     "changed", G_CALLBACK (device_selection_changed_cb), pp);
 
   actualize_devices_list (pp);
+
+  if (dbus_method_available (FIREWALLD_BUS,
+                             FIREWALLD_PATH,
+                             FIREWALLD_IFACE,
+                             "getServices"))
+    {
+      if (!service_enabled ("mdns"))
+        service_enable ("mdns", 300);
+
+      if (!service_enabled ("ipp"))
+        service_enable ("ipp", 300);
+
+      if (!service_enabled ("ipp-client"))
+        service_enable ("ipp-client", 300);
+
+      if (!service_enabled ("samba-client"))
+        service_enable ("samba-client", 300);
+    }
+  else
+    {
+      pp->warning = g_strdup (_("FirewallD is not running. \
+Network printer detection needs services mdns, ipp, ipp-client \
+and samba-client enabled on firewall."));
+
+      warning_textview = (GtkTextView*)
+        gtk_builder_get_object (pp->builder, "warning-textview");
+      text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (warning_textview));
+
+      gtk_text_buffer_set_text (text_buffer, "", 0);
+      gtk_text_buffer_get_iter_at_offset (text_buffer, &text_iter, 0);
+      gtk_text_buffer_insert (text_buffer, &text_iter, pp->warning, -1);
+
+      pp->show_warning = TRUE;
+    }
+
   devices_get (pp);
 
   renderer = gtk_cell_renderer_text_new ();
@@ -996,6 +1365,28 @@ dialog_closed (GtkWidget          *dialog,
   gtk_widget_destroy (dialog);
 }
 
+static GList *
+glist_uniq (GList *list)
+{
+  GList *result = NULL;
+  GList *iter = NULL;
+  GList *tmp = NULL;
+
+  for (iter = list; iter; iter = iter->next)
+    {
+      if (tmp == NULL ||
+          g_strcmp0 ((gchar *) tmp->data, (gchar *) iter->data) != 0)
+        {
+          tmp = iter;
+          result = g_list_append (result, g_strdup (iter->data));
+        }
+    }
+
+  g_list_free_full (list, g_free);
+
+  return result;
+}
+
 static void
 new_printer_add_button_cb (GtkButton *button,
                            gpointer   user_data)
@@ -1119,12 +1510,9 @@ new_printer_add_button_cb (GtkButton *button,
       else if (pp->devices[device_id].device_id)
         {
           /* Try whether CUPS has a driver for the new printer */
-          ppd_name = get_ppd_name (pp->devices[device_id].device_class,
-                           pp->devices[device_id].device_id,
-                           pp->devices[device_id].device_info,
-                           pp->devices[device_id].device_make_and_model,
-                           pp->devices[device_id].device_uri,
-                           pp->devices[device_id].device_location);
+          ppd_name = get_ppd_name (pp->devices[device_id].device_id,
+                       pp->devices[device_id].device_make_and_model,
+                       pp->devices[device_id].device_uri);
 
           if (ppd_name == NULL || ppd_name->ppd_match_level < PPD_EXACT_MATCH)
             {
@@ -1134,7 +1522,7 @@ new_printer_add_button_cb (GtkButton *button,
 
               proxy = get_dbus_proxy (PACKAGE_KIT_BUS,
                                       PACKAGE_KIT_PATH,
-                                      PACKAGE_KIT_IFACE,
+                                      PACKAGE_KIT_MODIFY_IFACE,
                                       FALSE);
 
               if (proxy)
@@ -1173,12 +1561,9 @@ new_printer_add_button_cb (GtkButton *button,
                     }
 
                   /* Search CUPS for driver */
-                  ppd_name = get_ppd_name (pp->devices[device_id].device_class,
-                               pp->devices[device_id].device_id,
-                               pp->devices[device_id].device_info,
+                  ppd_name = get_ppd_name (pp->devices[device_id].device_id,
                                pp->devices[device_id].device_make_and_model,
-                               pp->devices[device_id].device_uri,
-                               pp->devices[device_id].device_location);
+                               pp->devices[device_id].device_uri);
 
                   g_free (device_ids);
                 }
@@ -1237,11 +1622,14 @@ new_printer_add_button_cb (GtkButton *button,
       /* Set some options of the new printer */
       if (success)
         {
+          const char *ppd_file_name = NULL;
           DBusGProxy *proxy;
           GError     *error = NULL;
           char       *ret_error = NULL;
           char       *locale = NULL;
 
+          ppd_file_name = cupsGetPPD (pp->devices[device_id].display_name);
+
           proxy = get_dbus_proxy (MECHANISM_BUS,
                                   "/",
                                   MECHANISM_BUS,
@@ -1283,7 +1671,6 @@ new_printer_add_button_cb (GtkButton *button,
 
               if (locale)
                 {
-                  const char  *ppd_file_name = NULL;
                   ppd_file_t  *ppd_file = NULL;
                   gchar      **value = NULL;
                   gchar       *paper_size;
@@ -1297,7 +1684,6 @@ new_printer_add_button_cb (GtkButton *button,
                   else
                     paper_size = g_strdup ("A4");
 
-                  ppd_file_name = cupsGetPPD (pp->devices[device_id].display_name);
                   if (ppd_file_name)
                     {
                       ppd_file = ppdOpenFile (ppd_file_name);
@@ -1324,7 +1710,6 @@ new_printer_add_button_cb (GtkButton *button,
                                 }
                           ppdClose (ppd_file);
                         }
-                      g_unlink (ppd_file_name);
                     }
 
                   if (value)
@@ -1352,6 +1737,224 @@ new_printer_add_button_cb (GtkButton *button,
                 }
               g_object_unref (proxy);
             }
+
+          if (pp->devices[device_id].device_uri &&
+              dbus_method_available (FIREWALLD_BUS,
+                                     FIREWALLD_PATH,
+                                     FIREWALLD_IFACE,
+                                     "getServices"))
+            {
+              if (g_str_has_prefix (pp->devices[device_id].device_uri, "dnssd:") ||
+                  g_str_has_prefix (pp->devices[device_id].device_uri, "mdns:"))
+                {
+                  show_notification (_("Opening firewall for mDNS connections"),
+                                     NULL,
+                                     "dialog-information-symbolic");
+                  service_disable ("mdns");
+                  service_enable ("mdns", 0);
+                }
+
+              if (g_strrstr (pp->devices[device_id].device_uri, "smb:") != NULL)
+                {
+                  show_notification (_("Opening firewall for Samba connections"),
+                                     NULL,
+                                     "dialog-information-symbolic");
+                  service_disable ("samba-client");
+                  service_enable ("samba-client", 0);
+                }
+
+              if (g_strrstr (pp->devices[device_id].device_uri, "ipp:") != NULL)
+                {
+                  show_notification (_("Opening firewall for IPP connections"),
+                                     NULL,
+                                     "dialog-information-symbolic");
+                  service_disable ("ipp");
+                  service_enable ("ipp", 0);
+                  service_disable ("ipp-client");
+                  service_enable ("ipp-client", 0);
+                }
+            }
+
+          if (ppd_file_name)
+            {
+              GDBusProxy *proxy;
+              GVariant   *input;
+              GVariant   *output;
+              GVariant   *array;
+              GList      *executables = NULL;
+              GList      *packages = NULL;
+
+              g_clear_error (&error);
+              error = NULL;
+
+              proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                                     G_DBUS_PROXY_FLAGS_NONE,
+                                                     NULL,
+                                                     SCP_BUS,
+                                                     SCP_PATH,
+                                                     SCP_IFACE,
+                                                     NULL,
+                                                     &error);
+
+              if (proxy)
+                {
+                  input = g_variant_new ("(s)",
+                                         ppd_file_name);
+
+                  output = g_dbus_proxy_call_sync (proxy,
+                                                   "MissingExecutables",
+                                                   input,
+                                                   G_DBUS_CALL_FLAGS_NONE,
+                                                   60000,
+                                                   NULL,
+                                                   &error);
+
+                  if (output && g_variant_n_children (output) == 1)
+                    {
+                      array = g_variant_get_child_value (output, 0);
+                      if (array)
+                        {
+                          for (i = 0; i < g_variant_n_children (array); i++)
+                            {
+                              executables = g_list_append (
+                                              executables,
+                                                g_strdup (g_variant_get_string (
+                                                  g_variant_get_child_value (array, i),
+                                                  NULL)));
+                            }
+                        }
+                    }
+
+                  if (output)
+                    g_variant_unref (output);
+                  g_variant_unref (input);
+                  g_object_unref (proxy);
+                }
+
+              if (proxy == NULL ||
+                  (error &&
+                   error->domain == G_DBUS_ERROR &&
+                   (error->code == G_DBUS_ERROR_SERVICE_UNKNOWN ||
+                    error->code == G_DBUS_ERROR_UNKNOWN_METHOD)))
+                g_warning ("Install system-config-printer which provides \
+DBus method \"MissingExecutables\" to find missing executables and filters.");
+
+              executables = g_list_sort (executables, (GCompareFunc) g_strcmp0);
+              executables = glist_uniq (executables);
+
+              if (executables)
+                {
+                  proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                                         G_DBUS_PROXY_FLAGS_NONE,
+                                                         NULL,
+                                                         PACKAGE_KIT_BUS,
+                                                         PACKAGE_KIT_PATH,
+                                                         PACKAGE_KIT_QUERY_IFACE,
+                                                         NULL,
+                                                         &error);
+
+                  if (proxy)
+                    {
+                      GList *exec_iter;
+
+                      for (exec_iter = executables; exec_iter; exec_iter = exec_iter->next)
+                        {
+                          input = g_variant_new ("(ss)", (gchar *) exec_iter->data, "");
+
+                          output = g_dbus_proxy_call_sync (proxy,
+                                                           "SearchFile",
+                                                           input,
+                                                           G_DBUS_CALL_FLAGS_NONE,
+                                                           60000,
+                                                           NULL,
+                                                           &error);
+
+                          if (error)
+                            g_warning ("%s", error->message);
+
+                          if (output)
+                            {
+                              gboolean  installed;
+                              gchar    *package;
+
+                              g_variant_get (output,
+                                             "(bs)",
+                                             &installed,
+                                             &package);
+                              if (!installed)
+                                packages = g_list_append (packages, g_strdup (package));
+                            }
+
+                          if (output)
+                            g_variant_unref (output);
+                          g_variant_unref (input);
+                        }
+
+                      g_object_unref (proxy);
+                    }
+
+                  g_list_free_full (executables, g_free);
+                }
+
+              packages = g_list_sort (packages, (GCompareFunc) g_strcmp0);
+              packages = glist_uniq (packages);
+
+              if (packages)
+                {
+                  proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                                         G_DBUS_PROXY_FLAGS_NONE,
+                                                         NULL,
+                                                         PACKAGE_KIT_BUS,
+                                                         PACKAGE_KIT_PATH,
+                                                         PACKAGE_KIT_MODIFY_IFACE,
+                                                         NULL,
+                                                         &error);
+
+                  if (proxy)
+                    {
+                      GVariantBuilder  array_builder;
+                      GList           *pkg_iter;
+
+                      g_variant_builder_init (&array_builder, G_VARIANT_TYPE ("as"));
+
+                      for (pkg_iter = packages; pkg_iter; pkg_iter = pkg_iter->next)
+                        g_variant_builder_add (&array_builder,
+                                               "s",
+                                               (gchar *) pkg_iter->data);
+
+                      input = g_variant_new ("(uass)",
+#ifdef GDK_WINDOWING_X11
+                        GDK_WINDOW_XID (gtk_widget_get_window (GTK_WIDGET (pp->dialog))),
+#else
+                        0,
+#endif
+                        &array_builder,
+                        "hide-finished");
+
+                      output = g_dbus_proxy_call_sync (proxy,
+                                                       "InstallPackageNames",
+                                                       input,
+                                                       G_DBUS_CALL_FLAGS_NONE,
+                                                       60000,
+                                                       NULL,
+                                                       &error);
+
+                      if (error)
+                        g_warning ("%s", error->message);
+
+                      if (output)
+                        g_variant_unref (output);
+                      g_variant_unref (input);
+
+                      g_object_unref (proxy);
+                    }
+
+                  g_list_free_full (packages, g_free);
+                }
+            }
+
+          if (ppd_file_name)
+            g_unlink (ppd_file_name);
         }
     }
 
@@ -1407,6 +2010,8 @@ pp_new_printer_dialog_new (GtkWindow            *parent,
   pp->user_data = user_data;
 
   pp->cancellable = NULL;
+  pp->warning = NULL;
+  pp->show_warning = FALSE;
 
   /* connect signals */
   g_signal_connect (pp->dialog, "delete-event", G_CALLBACK (gtk_widget_hide_on_delete), NULL);
@@ -1459,6 +2064,8 @@ pp_new_printer_dialog_free (PpNewPrinterDialog *pp)
       g_object_unref (pp->cancellable);
     }
 
+  g_free (pp->warning);
+
   g_free (pp);
 }
 
