@@ -45,9 +45,8 @@
 
 #include "um-strength-bar.h"
 #include "um-editable-button.h"
-#include "um-editable-entry.h"
+#include "cc-editable-entry.h"
 #include "um-editable-combo.h"
-#include "libgnome-control-center/cc-lockbutton.h"
 
 #include "um-account-dialog.h"
 #include "um-language-dialog.h"
@@ -68,15 +67,12 @@ struct _UmUserPanelPrivate {
         GtkBuilder *builder;
 
         GtkWidget *main_box;
-        GtkWidget *lock_button;
         GPermission *permission;
         GtkWidget *language_chooser;
 
         UmAccountDialog *account_dialog;
         UmPasswordDialog *password_dialog;
         UmPhotoDialog *photo_dialog;
-
-        PolkitAuthority *authority;
 };
 
 static GtkWidget *
@@ -570,7 +566,7 @@ show_user (UmUser *user, UmUserPanelPrivate *d)
         um_photo_dialog_set_user (d->photo_dialog, user);
 
         widget = get_widget (d, "full-name-entry");
-        um_editable_entry_set_text (UM_EDITABLE_ENTRY (widget), um_user_get_real_name (user));
+        cc_editable_entry_set_text (CC_EDITABLE_ENTRY (widget), um_user_get_real_name (user));
         gtk_widget_set_tooltip_text (widget, um_user_get_user_name (user));
 
         widget = get_widget (d, "account-type-combo");
@@ -621,9 +617,7 @@ selected_user_changed (GtkTreeSelection *selection, UmUserPanelPrivate *d)
         if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
                 gtk_tree_model_get (model, &iter, USER_COL, &user, -1);
                 show_user (user, d);
-                if (d->lock_button != NULL) {
-                        on_permission_changed (d->permission, NULL, d);
-                }
+                on_permission_changed (d->permission, NULL, d);
                 g_object_unref (user);
         }
 }
@@ -637,7 +631,7 @@ change_name_done (GtkWidget          *entry,
 
         user = get_selected_user (d);
 
-        text = um_editable_entry_get_text (UM_EDITABLE_ENTRY (entry));
+        text = cc_editable_entry_get_text (CC_EDITABLE_ENTRY (entry));
 
         if (g_strcmp0 (text, um_user_get_real_name (user)) != 0) {
                 um_user_set_real_name (user, text);
@@ -960,7 +954,7 @@ on_permission_changed (GPermission *permission,
                 gtk_widget_show (get_widget (d, "user-icon-button"));
                 gtk_widget_hide (get_widget (d, "user-icon-nonbutton"));
 
-                um_editable_entry_set_editable (UM_EDITABLE_ENTRY (get_widget (d, "full-name-entry")), TRUE);
+                cc_editable_entry_set_editable (CC_EDITABLE_ENTRY (get_widget (d, "full-name-entry")), TRUE);
                 remove_unlock_tooltip (get_widget (d, "full-name-entry"));
 
                 um_editable_combo_set_editable (UM_EDITABLE_COMBO (get_widget (d, "account-language-combo")), TRUE);
@@ -975,7 +969,7 @@ on_permission_changed (GPermission *permission,
                 gtk_widget_hide (get_widget (d, "user-icon-button"));
                 gtk_widget_show (get_widget (d, "user-icon-nonbutton"));
 
-                um_editable_entry_set_editable (UM_EDITABLE_ENTRY (get_widget (d, "full-name-entry")), FALSE);
+                cc_editable_entry_set_editable (CC_EDITABLE_ENTRY (get_widget (d, "full-name-entry")), FALSE);
                 add_unlock_tooltip (get_widget (d, "full-name-entry"));
 
                 um_editable_combo_set_editable (UM_EDITABLE_COMBO (get_widget (d, "account-language-combo")), FALSE);
@@ -1087,7 +1081,6 @@ setup_main_window (UmUserPanelPrivate *d)
         GtkWidget *button;
         GtkTreeIter iter;
         gint expander_size;
-        GtkWidget *box;
         gchar *title;
         GIcon *icon;
         gchar *names[3];
@@ -1196,18 +1189,9 @@ setup_main_window (UmUserPanelPrivate *d)
                           G_CALLBACK (change_fingerprint), d);
 
         d->permission = (GPermission *)polkit_permission_new_sync ("org.freedesktop.accounts.user-administration", NULL, NULL, NULL);
-        if (d->permission != NULL) {
-                /* accounts service not available? */
-                button = cc_lock_button_new (d->permission);
-                gtk_widget_set_margin_top (button, 12);
-                gtk_widget_show (button);
-                box = get_widget (d, "accounts-vbox");
-                gtk_box_pack_end (GTK_BOX (box), button, FALSE, FALSE, 0);
-                g_signal_connect (d->permission, "notify",
-                                  G_CALLBACK (on_permission_changed), d);
-                on_permission_changed (d->permission, NULL, d);
-                d->lock_button = button;
-        }
+        g_signal_connect (d->permission, "notify",
+                          G_CALLBACK (on_permission_changed), d);
+        on_permission_changed (d->permission, NULL, d);
 
         button = get_widget (d, "add-user-toolbutton");
         names[0] = "changes-prevent-symbolic";
@@ -1244,14 +1228,13 @@ um_user_panel_init (UmUserPanel *self)
         /* register types that the builder might need */
         type = um_strength_bar_get_type ();
         type = um_editable_button_get_type ();
-        type = um_editable_entry_get_type ();
+        type = cc_editable_entry_get_type ();
         type = um_editable_combo_get_type ();
 
         gtk_widget_set_size_request (GTK_WIDGET (self), -1, 350);
 
         d->builder = gtk_builder_new ();
         d->um = um_user_manager_ref_default ();
-        d->authority = polkit_authority_get_sync (NULL, NULL);
 
         filename = UIDIR "/user-accounts-dialog.ui";
         if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
@@ -1307,21 +1290,32 @@ um_user_panel_dispose (GObject *object)
                 gtk_widget_destroy (priv->language_chooser);
                 priv->language_chooser = NULL;
         }
-        if (priv->authority) {
-                g_object_unref (priv->authority);
-                priv->authority = NULL;
+        if (priv->permission) {
+                g_object_unref (priv->permission);
+                priv->permission = NULL;
         }
 
         G_OBJECT_CLASS (um_user_panel_parent_class)->dispose (object);
 }
 
+static GPermission *
+um_user_panel_get_permission (CcPanel *panel)
+{
+        UmUserPanelPrivate *priv = UM_USER_PANEL (panel)->priv;
+
+        return priv->permission;
+}
+
 static void
 um_user_panel_class_init (UmUserPanelClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
+        CcPanelClass *panel_class = CC_PANEL_CLASS (klass);
 
         object_class->dispose = um_user_panel_dispose;
 
+        panel_class->get_permission = um_user_panel_get_permission;
+
         g_type_class_add_private (klass, sizeof (UmUserPanelPrivate));
 }
 
