@@ -61,7 +61,8 @@ enum
   COL_PIXBUF,
   COL_CATEGORY,
   COL_SEARCH_TARGET,
-  COL_ICON_NAME,
+  COL_GICON,
+  COL_KEYWORDS,
 
   N_COLS
 };
